/*
 * soelim - eliminate .so lines from troff documents
 *
 * This simple utility is provided to eliminate the .so lines
 * from troff documents by merging in the text from the referenced
 * source file.  Some systems already come with soelim, others do not.
 *
 * Usage: soelim [-d include [-d include]...] inputfile
 *	The named input file is opened processed.  Any .so line in the
 *	input is replace by the lines from the file.
 *
 *	Output is written to standard out
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>

char *sodir[] = { 0, 0, 0, 0, 0 };

main(argc, argv)
	int argc;
	char *argv[];
{
	int   dofile();
	int   dirct = 0;
	FILE *filein;
	int   i;
	extern int   optind;
	extern char *optarg;

	while ((i = getopt(argc, argv, "d:")) != -1) 
		switch (i) {
		    case (int)'d':	sodir[dirct++] = optarg;
					break;

		    default:	fprintf(stderr, "Usage: %s [-d include_dir] [filename ...]\n", argv[0]);
				return 1;
		}

	if (optind < argc) {
		for (i=optind; i<argc; ++i) {
			
			filein = fopen(argv[i], "r");
			if (filein == NULL) {
				fprintf(stderr, "soelim: unable to open %s\n", argv[i]);
				return (1);
			}
			if (dofile(filein) != 0)
				return (1);
		}
	} else {
		return ( dofile(stdin) );
	}
	return (0);
}

int dofile(filein)
	FILE *filein;
{
	char buf[256];
	int  dirindex;
	int  found;
	char newfile[1024];
	FILE *newin;
	char *pc;
	char *pdir;
	char *pso;

	while (fgets(buf, 255, filein) != NULL) {
		if (strncmp(buf, ".so ", 4) == 0) {
			dirindex = 0;
			found = 0;
			pso = buf + 4;
			while (*pso == ' ')
				pso++;
			pc = pso;
			while (isspace((int)*pc) == 0)
				pc++;
			*pc = '\0';
			while (pdir = sodir[dirindex] ) {
				(void)strcpy(newfile, pdir);
				(void)strcat(newfile, "/");
				(void)strcat(newfile, pso);

				if ((newin = fopen(newfile, "r")) != NULL) {
					found = 1;
					dofile(newin);
					break;
				}
				dirindex++;
			}
			if (found == 0) {
				/* try local directory */

				(void)strcpy(newfile, pso);
				if ((newin = fopen(newfile, "r")) != NULL) {
					dofile(newin);
				} else {
					fprintf(stderr, "Unable to find include file %s\n", pso);
					exit(1);
				}
			}

		} else {
			fputs(buf, stdout);
		}
	}
	fclose(filein);
	return (0);
}
